#ifndef AG_DICTIONARY_BASE_H
#define AG_DICTIONARY_BASE_H

/*
 * The AGCLIB1 Class Library, Version 1.0
 * Copyright (c) Parsifal Software, 2001. All Rights Reserved.
 * http://www.parsifalsoft.com
*/

#include "agmap.h"
#include "agstk.h"

template <class Object>
class AgDictionaryBase
: public AgReferenceCounter
{
protected:
  AgMapBase<Object,int> indexMap;
  AgStackBase<Object> objectStore;
public:
  AgDictionaryBase()
    : AgReferenceCounter(), indexMap(), objectStore()  {}
  AgDictionaryBase(const AgDictionaryBase<Object> &d);
  AgDictionaryBase(const AgContainerContents<Object> &);
  ~AgDictionaryBase() {}
  AgDictionaryBase<Object> &reset() {indexMap.reset(); objectStore.reset(); return *this;}
  int lookup(const Object &s) const;
  int intern(const Object &s);
  const Object &operator [](int x) const {return objectStore[x];}
  int size() const {return objectStore.size();}
  int operator == (const AgDictionaryBase<Object> &x) const {return objectStore == x.objectStore;}
  int operator != (const AgDictionaryBase<Object> &x) const {return objectStore != x.objectStore;}
};

template <class Object>
AgDictionaryBase<Object>::AgDictionaryBase(const AgContainerContents<Object> &c)
: AgReferenceCounter()
{
  int i = 0;
  while (i < c.size()) intern(c[i++]);
}

template <class Object>
AgDictionaryBase<Object>::AgDictionaryBase(const AgDictionaryBase<Object> &d)
  : AgReferenceCounter(),
    indexMap(d.indexMap),
    objectStore(d.objectStore)
{}

template <class Object>
int AgDictionaryBase<Object>::lookup(const Object &object) const {
  const int *pIndex = indexMap.pValue(object);
  return pIndex ? *pIndex : -1;
}

template <class Object>
int AgDictionaryBase<Object>::intern(const Object &object) {
  int *pIndex = indexMap.pValue(object);
  if (pIndex) return *pIndex;
  int index = indexMap.value(object) = objectStore.size();
  objectStore.push(object);
  return index;
}

#endif
