#ifndef AG_MAP_H
#define AG_MAP_H

/*
 * The AGCLIB1 Class Library, Version 1.0
 * Copyright (c) Parsifal Software, 2001. All Rights Reserved.
 * http://www.parsifalsoft.com
*/

#include <stdlib.h>
#include "agbmap.h"
#include "agref.h"

template <class Key, class Value>
class AgMap
: public AgCopyControl< AgMapBase<Key,Value> >
{
  typedef AgKVPair<Key, Value> Pair;
  typedef AgMapBase<Key, Value> MapBase;
  typedef AgMap<Key, Value> This;
  typedef AgCopyControl<MapBase> Base;
public:
  typedef AgContents<Pair, Base> Contents;
  AgMap() : Base(new MapBase()) {}
  AgMap(const This &x) : Base(x) {}
  AgMap(const AgContainerContents<Pair> &c) : Base(new MapBase(c)) {}
  ~AgMap() {}

  Value &value(const Key &key) {return kernel()->value(key);}

  Value *pValue(const Key &key) {return kernel()->pValue(key);}
  const Value *pValue(const Key &key) const {return kernel()->pValue(key);}

  int remove(const Key &key) {return kernel()->remove(key);}
  int size() const {return kernel()->size();}
  This &reset() {kernel()->reset(); return *this;}
  const Contents contents() const {return Contents(*this);}

  class Keys  : public AgContainerContents<Key> {
    const AgCopyControl< AgMapBase<Key, Value> > map;
  public:
    Keys(const AgMap<Key, Value> &m) : AgContainerContents<Key>(), map(m) {}
    const Key &operator [](int x) const {return map.kernel()->operator [](x).key();}
    int size() const {return map.kernel()->size();}
  };

  class Values : public AgContainerContents<Value> {
    const AgCopyControl< AgMapBase<Key, Value> > map;
  public:
    Values(const AgMap<Key, Value> &m) : AgContainerContents<Value>(), map(m) {}
    const Value &operator [](int x) const {return map.kernel()->operator [](x).value();}
    int size() const {return map.kernel()->size();}
  };
  const Keys keys() const {return Keys(*this);}
  const Values values() const {return Values(*this);}

  This &operator = (const This &x) {Base::operator = (x); return *this;}
  This &operator = (const AgContainerContents<Pair> &x) {*kernel() = x; return *this;}

  Pair &operator [](int x) {return kernel()->operator[](x);}
  const Pair &operator [](int x) const {return kernel()->operator[](x);}

  //int operator == (const AgMap<Key, Value> &x) const {return *kernel() == *x.kernel();}
  //int operator != (const AgMap<Key, Value> &x) const {return *kernel() != *x.kernel();}
};

template <class Key, class Value, class Container>
AgMap<Key, Value> &operator += (AgMap<Key, Value> &m, const Container &c) {
  m.reset();
  int n = c.size();
  for (int i = 0; i < n; i++) {
    const AgKVPair<Key, Value> &pair = (const AgKVPair<Key, Value> &) c[i];
    m.value(pair.key()) = pair.value();
  }
  return m;
}

/*
#include "agset.h"

template <class Object>
AgMap<Object, AgSet<Object> > transitiveClosure(const AgMap<Object, Object> &map) {
  AgMap<Object, AgSet<Object> > closure;
  int n = map.size();
  for (int i = 0; i < n; i++) {
    Object *object = &map[i].key();
    AgSet<Object> &set = closure.value(*object);
    Object *value = map.pValue(*object);
    do {
      set.add(*value);
      value = map.pValue(*value);
    } while (value);
  }
  return closure;
}

template <class Key, class Value>
AgMap<Value, AgSet<Key> > inverse(const AgMap<Key, Value> &map) {
  AgMap<Value, AgSet<Key> > inverseMap;
  for (int i = 0; i < map.size(); i++) {
    const AgKVPair<Key, Value> &pair = map[i];
    inverseMap(pair.value()).value() = pair.key();
  }
  return inverseMap;
}
*/

template <class Key, class Value>
int agABTHash(const AgMap<Key, Value> &m, int startValue) {
  return agABTHash(m.kernel()->hash(), startValue);
}

template <class Key, class Value>
int agABTHash(const AgMap<Key, Value> &m) {
  return agABTHash(m, 0);
}

#endif
