#ifndef AG_STACK_H
#define AG_STACK_H

/*
 * The AGCLIB1 Class Library, Version 1.0
 * Copyright (c) Parsifal Software, 2001. All Rights Reserved.
 * http://www.parsifalsoft.com
*/

#include "agbstk.h"

template <class Object>
class AgStack
: public AgCopyControl<AgStackBase <Object> >
{
  typedef AgStackBase<Object>        StackBase;
  typedef AgCopyControl<StackBase>   Base;
  typedef AgStack<Object>            This;
public:
  typedef AgContents<Object, Base> Contents;
  AgStack(const int bufferSize = DEFAULT_BUFFER_SIZE) : Base(new StackBase(bufferSize)) {}
  AgStack(const AgContainerContents<Object> &x, int bufferSize = DEFAULT_BUFFER_SIZE) : Base(new StackBase(x, bufferSize)) {}
  AgStack(const This &s) : Base(s) {}
  ~AgStack() {}

  This &push(const Object &x) {kernel()->push(x); return *this;}
  Object pop() {return kernel()->pop();}

  const Object &top() const {return kernel()->top();}
  Object &top() {return kernel()->top();}

  int size() const { return kernel()->size();}
  This &reset() {kernel()->reset(); return *this;}
  const Contents contents() const {return Contents(*this);}

  This &operator = (const This &x) {Base::operator = (x); return *this;}
  This &operator = (const AgContainerContents<Object> &x) {*kernel() = x; return *this;}

  Object &operator [] (int x) {return kernel()->operator[] (x);}
  const Object &operator [] (int x) const {return kernel()->operator[] (x);}
};

template <class Object, class Container>
AgStack<Object> &operator += (AgStack<Object> &s, const Container &c) {
  int n = c.size();
  for (int i = 0; i < n; i++) s.push((Object &) c[i]);
  return s;
}

template <class Object>
int agABTHash(const AgStack<Object> &c, int hash) {
  return agABTHashContainer(c, hash);
}

template <class Object>
inline int agABTHash(const AgStack<Object> &c) {
  return agABTHashContainer(c, 0);
}

#endif
